@@ -11,7 +11,7 @@ Apache License 2.0
 See COPYING for more information.
 '''
 __author__ = 'Luke Campbell'
-__version__ = '0.4'
+__version__ = '0.5'
 
 
 import json
@@ -137,7 +137,7 @@ class ElasticSearch(object):
             print content
         url_request = urllib2.Request(url,content)
         url_request.add_header('Content-Type','application/json')
-        url_request.get_method = lambda : 'PUT'
+        url_request.get_method = lambda : 'POST'
         s = urllib2.urlopen(url_request).read()
         return json.loads(s)
 
@@ -237,7 +237,7 @@ class ElasticSearch(object):
         s = urllib2.urlopen(url_request).read()
         return json.loads(s)
 
-    def river_couchdb_create(self, index_name,index_type='',couchdb_db='', river_name='',couchdb_host='localhost', couchdb_port='5984',couchdb_filter=None,script=''):
+    def river_couchdb_create(self, index_name,index_type='',couchdb_db='', river_name='',couchdb_host='localhost', couchdb_port='5984',couchdb_user=None, couchdb_password=None, couchdb_filter=None,script=''):
         '''
         https://github.com/elasticsearch/elasticsearch-river-couchdb
 
@@ -261,13 +261,16 @@ class ElasticSearch(object):
                     'host' : couchdb_host,
                     'port' : couchdb_port,
                     'db' : couchdb_db,
-                    'filter' : couch_filter
+                    'filter' : couchdb_filter
                     },
                 'index' : {
                     'index' : index_name,
                     'type' : index_type
                     }
                 }
+        if couchdb_user and couchdb_password:
+            content['couchdb']['user'] = couchdb_user
+            content['couchdb']['password'] = couchdb_password
         if script:
             content['couchdb']['script'] = script
         content_json = json.dumps(content)
@@ -354,7 +357,32 @@ class ElasticSearch(object):
         request.get_method = lambda : 'PUT'
         s = urllib2.urlopen(request).read()
         return json.loads(s)
-        
+
+    def type_list(self, index_name):
+        '''
+        List the types available in an index
+        '''
+        url = 'http://%s:%s/%s/_mapping' % (self.host, self.port, index_name)
+        request = urllib2.Request(url, None)
+        request.add_header('Content-Type','application/json')
+        if self.verbose:
+            print url
+        response = json.loads(urllib2.urlopen(request).read())
+        return response[index_name].keys()
+
+    def raw(self, module, method, data):
+        '''
+        Submits or requsts raw input
+        '''
+        url = 'http://%s:%s/%s' % (self.host, self.port, module)
+        content = json.dumps(data)
+        if self.verbose:
+            print content
+        request = urllib2.Request(url,content)
+        request.add_header('Content-Type','application/json')
+        request.get_method = lambda : method
+        response = json.loads(urllib2.urlopen(request).read())
+        return response
 
 class ElasticQuery(dict):
     '''
